# spreadsheet-use: Alias package for univer-use
# This package provides the same functionality as univer-use
from univer_use.spreadsheet_act import spreadsheet_agent
from univer_use.graph import build_graph
from univer_use import *

__all__ = ["spreadsheet_agent", "build_graph"]